searchNodes=[{"type":"module","title":"Mneme","doc":"/ˈniːmiː/ - Snapshot testing utilities Mneme provides a set of familiar assertions that automate the tedious parts of testing. This is sometimes called snapshot testing or approval testing, but that's not particulary important. With Mneme, you write something like auto_assert my_function ( ) and next time you run your tests, Mneme runs the function, generates a pattern from the returned value, prompts you to confirm it's what you expected, and updates the code for you. Now you have: auto_assert % MyAwesomeValue { so : :cool } &lt;- my_function ( ) This lets you quickly write lots of tests to ensure the behavior of your program doesn't change without you knowing. And if it does, Mneme prompts you with a diff so you can easily see what's up. Features: Automatically-maintained assertions: compare values using auto_assert , test exceptions using auto_assert_raise , or test process messages using auto_assert_receive and friends. Seamless integration with ExUnit: no need to change your workflow, just run mix test . Interactive prompts in your terminal when a new assertion is added or an existing one changes. Syntax-aware diffs highlight the meaningful changes in a value. Pattern matching auto-assertions let you assert on the parts of a value that matter and ignore the rest. Take a brief tour: If you'd like to see Mneme in action, you can download and run examples/tour_mneme.exs , a standalone tour that only requires that you have Elixir installed. $ curl -o tour_mneme.exs https://raw.githubusercontent.com/zachallaun/mneme/main/examples/tour_mneme.exs $ elixir tour_mneme.exs Setup Add :mneme do your deps in mix.exs : defp deps do [ { :mneme , &quot;&gt;= 0.0.0&quot; , only : :test } ] end Add :mneme to your :import_deps in .formatter.exs : [ import_deps : [ :mneme ] , inputs : [ &quot;{mix,.formatter}.exs&quot; , &quot;{config,lib,test}/**/*.{ex,exs}&quot; ] ] Start Mneme right after you start ExUnit in test/test_helper.exs : ExUnit . start ( ) Mneme . start ( ) Add use Mneme wherever you use ExUnit.Case : defmodule MyTest do use ExUnit.Case , async : true use Mneme test &quot;arithmetic&quot; do # use auto_assert instead of ExUnit&#39;s assert - run this test # and delight in all the typing you don&#39;t have to do auto_assert 2 + 2 end end Generated patterns Mneme tries to generate match patterns that are close to what you would write. Basic data types like numbers, lists, tuples, and strings will generate as you would expect. For maps and structs, Mneme will often give you a couple of options. For values without a literal representation, like pids, guards will be used. auto_assert pid when is_pid ( pid ) &lt;- self ( ) Additionally, local bindings can be found and pinned as a part of patterns. This keeps the number of &quot;magic values&quot; down and makes tests more robust. test &quot;create_post/1 creates a new post with valid attrs&quot; , %{ user : user } do valid_attrs = %{ title : &quot;my_post&quot; , author : user } auto_assert create_post ( valid_attrs ) end # generates: test &quot;create_post/1 creates a new post with valid attrs&quot; , %{ user : user } do valid_attrs = %{ title : &quot;my_post&quot; , author : user } auto_assert { :ok , % Post { title : &quot;my_post&quot; , author : ^ user } } &lt;- create_post ( valid_attrs ) end Non-exhaustive list of special cases Pinned variables are generated by default if a value is equal to a variable in scope. Date and time values are written using their sigil representation. Struct patterns only include fields that are different from the struct defaults. Structs defined by Ecto schemas exclude primary keys, association foreign keys, and auto generated fields like :inserted_at and :updated_at . This is because these fields are often randomly generated and would fail on subsequent tests. Formatting required Mneme uses Rewrite to update source code, formatting that code before saving the file. Currently, the Elixir formatter and FreedomFormatter are supported. If you do not use a formatter, the first auto-assertion will reformat the entire file. Continuous Integration In a CI environment, Mneme will not attempt to prompt and update any assertions, but will instead fail any tests that would update. This behavior is enabled by the CI environment variable, which is set by convention by many continuous integration providers. export CI=true Editor support Guides for optional editor integration can be found here: VS Code Configuration Mneme supports a variety of flexible configuration options that can be applied at multiple levels of granularity, from your entire test suite down to an individual test. While the default behavior will work well for the majority of cases, it's worth knowing which levers and knobs you have available to tweak Mneme to fit your individual workflow. Options :action ( :prompt | :accept | :reject ) - The action to be taken when an auto-assertion updates. If CI=true is set in environment variables, the action will always be :reject . The default value is :prompt . :default_pattern ( :infer | :first | :last ) - The default pattern to be selected if prompted to update an assertion. The default value is :infer . :diff ( :text | :semantic ) - Controls the diff engine used to display changes when an auto- assertion updates. If :semantic , uses a custom diff engine to highlight only meaningful changes in the value. If :text , uses the Myers Difference algorithm to highlight all changes in text. The default value is :semantic . :diff_style ( :side_by_side | :stacked ) - Controls how diffs are rendered when the :diff option is set to :semantic . If :side_by_side , old and new code will be rendered side-by-side if the terminal has sufficient space. If :stacked , old and new code will be rendered one on top of the other. The default value is :side_by_side . :force_update ( boolean/0 ) - Setting to true will force auto-assertions to update even when they would otherwise succeed. This can be especially helpful when adding new keys to maps or structs since a pattern like %{} would not normally prompt as the match still succeeds. The default value is false . :target ( :mneme | :ex_unit ) - The target output for auto-assertions. If :mneme , the expression will remain an auto-assertion. If :ex_unit , the expression will be rewritten as an ExUnit assertion. The default value is :mneme . Configuring Mneme There are four ways you can apply configuration options; Each is more specific than the last and will override any conflicting options that were set prior. When calling Mneme.start/1 , which will apply to the entire test run; When calling use Mneme , which will apply to all tests in that module; In a @mneme_describe module attribute, which will apply to all tests that follow in the given ExUnit.Case.describe/2 block; In a @mneme module attribute, which will apply only to the next test that follows. For instance, when an auto-assertion has multiple possible patterns available, Mneme will try to infer the best one to show you first. If you always want the last (and usually most complex) generated pattern, you could call Mneme.start/1 like this: # test/test_helper.exs ExUnit . start ( ) Mneme . start ( default_pattern : :last ) As mentioned above, this can be overriden at the module-level, in a describe block, or for an individual test: defmodule MyTest do use ExUnit.Case use Mneme , default_pattern : :infer test &quot;the default pattern will exclude :baz when this runs&quot; do map = %{ foo : :one , baz : :three } auto_assert %{ foo : 1 , bar : 2 } &lt;- Map . put ( map , bar : :two ) end describe &quot;...&quot; do @mneme_describe action : :reject test &quot;fails without prompting&quot; do auto_assert :wrong &lt;- MyModule . some_fun ( ) end @mneme action : :prompt test &quot;prompts to update&quot; do auto_assert :wrong &lt;- MyModule . another_fun ( ) end end end Breaking up with Mneme? While the official stance of the library is that this is Not Recommended™, Mneme can even convert all of your existing auto-assertions to regular assertions: Mneme . start ( target : :ex_unit , force_update : true , action : :accept ) Probably don't do this, but if you do, make sure all your tests are committed first in case you want to get back together. use Mneme When you use Mneme in a test module, assertions are imported and module attributes are made available for configuration .","ref":"Mneme.html"},{"type":"macro","title":"Mneme.auto_assert/1","doc":"Pattern-generating variant of ExUnit.Assertions.assert/1 . Examples auto_assert generates assertions when tests run, issuing a terminal prompt before making any changes (unless configured otherwise). auto_assert [ 1 , 2 ] ++ [ 3 , 4 ] # after running the test and accepting the change auto_assert [ 1 , 2 , 3 , 4 ] &lt;- [ 1 , 2 ] ++ [ 3 , 4 ] If the match no longer succeeds, a warning and new prompt will be issued to update it to the new value. auto_assert [ 1 , 2 , 3 , 4 ] &lt;- [ 1 , 2 ] ++ [ :a , :b ] # after running the test and accepting the change auto_assert [ 1 , 2 , :a , :b ] &lt;- [ 1 , 2 ] ++ [ :a , :b ] Prompts are only issued if the pattern doesn't match the value, so that pattern can also be changed manually. # this assertion succeeds, so no prompt is issued auto_assert [ 1 , 2 , | _ ] &lt;- [ 1 , 2 ] ++ [ :a , :b ] Differences from ExUnit assert The auto_assert macro is meant to match assert very closely, but there are a few differences to note: Pattern-matching assertions use the &lt;- operator instead of the = match operator. Unlike ExUnit's assert , auto_assert can match falsy values. The following are equivalent: falsy = nil auto_assert nil &lt;- falsy assert falsy == nil Guards can be added with a when clause, while assert would require a second assertion. For example: auto_assert pid when is_pid ( pid ) &lt;- self ( ) assert pid = self ( ) assert is_pid ( pid ) Bindings in an auto_assert are not available outside of that assertion. For example: auto_assert pid when is_pid ( pid ) &lt;- self ( ) pid # ERROR: pid is not bound If you need to use the result of the assertion, it will evaluate to the expression's value. pid = auto_assert pid when is_pid ( pid ) &lt;- self ( ) pid # pid is the result of self()","ref":"Mneme.html#auto_assert/1"},{"type":"macro","title":"Mneme.auto_assert_raise/1","doc":"See auto_assert_raise/3 .","ref":"Mneme.html#auto_assert_raise/1"},{"type":"macro","title":"Mneme.auto_assert_raise/2","doc":"See auto_assert_raise/3 .","ref":"Mneme.html#auto_assert_raise/2"},{"type":"macro","title":"Mneme.auto_assert_raise/3","doc":"Pattern-generating variant of ExUnit.Assertions.assert_raise/3 . If the given function does not raise, the assertion will fail. Like auto_assert/1 , you will be prompted to automatically update the assertion if the raised raised exception changes. Examples You can pass an anonymous function that takes no arguments and is expected to raise an exception. auto_assert_raise fn -&gt; some_call_expected_to_raise ( ) end # after running the test and accepting changes auto_assert_raise Some.Exception , fn -&gt; some_call_expected_to_raise ( ) end # optionally include the message auto_assert_raise Some.Exception , &quot;perhaps with a message&quot; , fn -&gt; some_call_expected_to_raise ( ) end A captured function of arity zero can also be used. auto_assert_raise &amp; some_call_expected_to_raise / 0 # after running the test and accepting changes auto_assert_raise Some.Exception , &amp; some_call_expected_to_raise / 0","ref":"Mneme.html#auto_assert_raise/3"},{"type":"macro","title":"Mneme.auto_assert_receive/0","doc":"See auto_assert_receive/2 .","ref":"Mneme.html#auto_assert_receive/0"},{"type":"macro","title":"Mneme.auto_assert_receive/1","doc":"See auto_assert_receive/2 .","ref":"Mneme.html#auto_assert_receive/1"},{"type":"macro","title":"Mneme.auto_assert_receive/2","doc":"Pattern-generating variant of ExUnit.Assertions.assert_receive/3 . timeout is in milliseconds and defaults to 100 . Examples Process . send_after ( self ( ) , { :some , :message } , 50 ) auto_assert_receive ( ) # after running the test, messages appearing within 100ms # will be available as options auto_assert_receive { :some , :message } A custom timeout can be specified as a second argument. Process . send_after ( self ( ) , { :some , :message } , 150 ) auto_assert_receive nil , 300 # messages appearing within 300ms will now appear as options auto_assert_receive { :some , :message } , 300","ref":"Mneme.html#auto_assert_receive/2"},{"type":"macro","title":"Mneme.auto_assert_received/0","doc":"See auto_assert_received/1 .","ref":"Mneme.html#auto_assert_received/0"},{"type":"macro","title":"Mneme.auto_assert_received/1","doc":"Pattern-generating variant of ExUnit.Assertions.assert_received/2 . Similar to auto_assert_receive/2 , except that the timeout is set to 0, so the expected message must already be in the current process' mailbox. Examples send ( self ( ) , { :some , :message } ) auto_assert_received ( ) # after running the test, messages in the current process # inbox will be available as options auto_assert_receive { :some , :message }","ref":"Mneme.html#auto_assert_received/1"},{"type":"function","title":"Mneme.start/1","doc":"Starts Mneme to run auto-assertions as they appear in your tests. This will almost always be added to your test/test_helper.exs , just below the call to ExUnit.start() : # test/test_helper.exs ExUnit . start ( ) Mneme . start ( ) Options :restart (boolean) - Restarts Mneme if it has previously been started. This option enables certain IEx-based testing workflows that allow tests to be run without a startup penalty. Defaults to false . Any option defined in the Configuration section of the module docs.","ref":"Mneme.html#start/1"},{"type":"exception","title":"Mneme.AssertionError","doc":"","ref":"Mneme.AssertionError.html"},{"type":"exception","title":"Mneme.CompileError","doc":"","ref":"Mneme.CompileError.html"},{"type":"exception","title":"Mneme.InternalError","doc":"","ref":"Mneme.InternalError.html"},{"type":"extras","title":"Changelog","doc":"This format is based on Keep a Changelog and this project adheres to Semantic Versioning .","ref":"changelog.html"},{"type":"extras","title":"Changelog - v0.4.0 (2023-07-04)","doc":"Added Mneme.start/1 now accepts all of Mneme's configuration options that will be applied to the entire test run. Removed Breaking: Removed support for :mneme application config. This has been replaced by passing config options directly to Mneme.start/1 : # Old: config/test.exs config :mneme , defaults : [ default_pattern : :last , ... ] # New: test/test_helper.exs Mneme . start ( default_pattern : :last , ... )","ref":"changelog.html#v0-4-0-2023-07-04"},{"type":"extras","title":"Changelog - v0.3.5 (2023-06-30)","doc":"Fixed Support multi-letter sigils when using Elixir 1.15.0+.","ref":"changelog.html#v0-3-5-2023-06-30"},{"type":"extras","title":"Changelog - v0.3.4 (2023-05-22)","doc":"Added Tested to support OTP 26.0 when using Elixir 1.14.4. Support expressions that return functions, serializing them with an is_function(fun, arity) guard. When generating a pattern for a MapSet, add a note suggesting using MapSet.to_list/1 for better serialization. Changed Format pattern notes to be more obvious when they're present. Generate charlist patterns using sigil_c instead of single quotes, e.g. ~c&quot;foo&quot; instead of 'foo' . See this discussion for more context. Fixed Numerous fixes related to vars used in guards: Generated vars will no longer shadow variables in scope (e.g. if pid is in scope, a different pid will use the var pid1 ). The same var will no longer be used for different values of the same type. Multiple, redundant guards will no longer be emitted for the same var (e.g. [self(), self()] would result in [pid, pid] when is_pid(pid) and is_pid(pid) ). Numerous fixes related to pattern generation, especially in regards to map keys.","ref":"changelog.html#v0-3-4-2023-05-22"},{"type":"extras","title":"Changelog - v0.3.3 (2023-05-01)","doc":"Changed Improve stacktraces and ExUnit assertion errors when an auto-assertion fails or is rejected. When an auto_assert updates, a default pattern that is more similar to the existing one will be selected in more cases when the :default_pattern option is set to :infer (the default). When updating an assertion with an existing map pattern that only asserts a subset of keys, generate a pattern using that subset as well.","ref":"changelog.html#v0-3-3-2023-05-01"},{"type":"extras","title":"Changelog - v0.3.2 (2023-04-16)","doc":"Changed Multi-line string patterns will now always appear in the same order (heredoc format will always be the last option). Fixed Raise a more comprehensible error if Mneme.start() is called multiple times without restart: true . Fix an incorrect guard that could cause semantic diffing to fail and fall back to text diffing. Fix multi-line string formatting issues with auto_assert_raise , auto_assert_receive , and auto_assert_received .","ref":"changelog.html#v0-3-2-2023-04-16"},{"type":"extras","title":"Changelog - v0.3.1 (2023-04-14)","doc":"Changed Multi-line strings will now generate both a heredoc and a single-line option. Removed No longer depend on libgraph .","ref":"changelog.html#v0-3-1-2023-04-14"},{"type":"extras","title":"Changelog - v0.3.0 (2023-04-10)","doc":"It is recommended to now use Elixir v1.14.4 or later. Added Add three new auto-assertions: auto_assert_raise auto_assert_receive auto_assert_received Add a :force_update option that forces re-generating assertion patterns even when they succeed. See the Options documentation for more info. Prompts will now show options that were overridden from the defaults. Mneme now prints a one-line summary at the end of the test run. Changed For falsy values, auto_assert now generates &lt;- pattern matches instead of == value comparisons, which have been removed. Existing auto-assertions will now run when Mneme.start() is not called, but new or failing auto-assertions will fail without prompting ( #32 ). Ranges now use range syntax like 1..10 and 1..10//2 instead of generating a %Range{} struct. Removed Breaking: auto_assert no longer supports value comparisons using == . Fixed Fix a configuration precedence bug that caused options set in application config to always override module, describe, or test options. Fix a compatibility issue with Ecto ~&gt; 3.9.4 ( #34 ). Fix a confusing diff result that could occur with some binary operations ( #11 ). Preceding comments are no longer shown in diffs ( #26 ). Fix a number of diffing errors related to structs.","ref":"changelog.html#v0-3-0-2023-04-10"},{"type":"extras","title":"Changelog - v0.2.7 (2023-03-29)","doc":"Fixed Fix a crash related to escaped string interpolation characters ( #29 ).","ref":"changelog.html#v0-2-7-2023-03-29"},{"type":"extras","title":"Changelog - v0.2.6 (2023-03-27)","doc":"Added Auto-assertion prompts can now be skipped ( s ) in addition to accepted ( y ) or rejected ( n ). This allows the test clause to continue so that later assertions might be run, but fails the test run once the suite finishes. Semantic diffs will now be displayed side-by-side if terminal width allows. To always display diffs stacked, use the diff_style: :stacked option; see the &quot;Configuration&quot; section of the Mneme module doc for more. Changed Semantic diff formatting has been improved for clarity. Fixed Don't overwrite test files if their content changes after starting the test run ( #23 ). Fix a crash that occurred when a value contained nested strings with newlines, e.g. {:ok, &quot;hello\\nworld&quot;} ( #25 ). The j/k options will no longer be rendered when prompting if there is only a single pattern option.","ref":"changelog.html#v0-2-6-2023-03-27"},{"type":"extras","title":"Changelog - v0.2.4, v0.2.5 (2023-03-25)","doc":"Fixed Remove unnecessary files from Hex package. This cuts the package size down drastically.","ref":"changelog.html#v0-2-4-v0-2-5-2023-03-25"},{"type":"extras","title":"Changelog - v0.2.3 (2023-03-25)","doc":"Fixed Fix diffing for certain sigil variations. Fix dbg -related error when running MIX_ENV=test iex -S mix ( #20 ). Fix ETS-related error when calling Mneme.start/1 multiple times ( #20 ).","ref":"changelog.html#v0-2-3-2023-03-25"},{"type":"extras","title":"Changelog - v0.2.2 (2023-03-20)","doc":"Fixed Disable a semantic diffing optimization that caused poor diff results in certain cases, usually manifesting as incorrect branches being compared.","ref":"changelog.html#v0-2-2-2023-03-20"},{"type":"extras","title":"Changelog - v0.2.1 (2023-03-19)","doc":"Changed More consistent formatting between :semantic and :text diffs.","ref":"changelog.html#v0-2-1-2023-03-19"},{"type":"extras","title":"Changelog - v0.2.0 (2023-03-18)","doc":"Added Adds semantic diffs which selectively highlight only meaningful changes when updating an assertion. This can be disabled with the diff: :text option; see the &quot;Configuration&quot; section of the Mneme module doc for more. Changed Breaking: Mneme now requires Elixir v1.14 or later. Fixed Invalid options now cause a warning instead of crashing test process. Internal errors now show an error instead of crashing test process. Fix bug causing multiple identical choices to be presented in some cases where empty lists were a part of the value.","ref":"changelog.html#v0-2-0-2023-03-18"},{"type":"extras","title":"Changelog - v0.1.6 (2023-03-04)","doc":"Changed Improved compile-time error message when auto_assert is used outside of a test block ( #9 ).","ref":"changelog.html#v0-1-6-2023-03-04"},{"type":"extras","title":"Changelog - v0.1.5 (2023-02-25)","doc":"Changed More consistent handling of charlists: lists of integers will now generate themselves as a pattern as well as a charlist if the list is ASCII printable ( #6 ).","ref":"changelog.html#v0-1-5-2023-02-25"},{"type":"extras","title":"Changelog - v0.1.4 (2023-02-23)","doc":"Fixed Fix a bug that could cause auto_assert expressions to revert to the previous value when using Mneme.start(restart: true) ( #7 ).","ref":"changelog.html#v0-1-4-2023-02-23"},{"type":"extras","title":"Changelog - v0.1.3 (2023-02-22)","doc":"Added Add a :default_pattern configuration option for auto-assertions which controls the pattern that should be selected by default when prompted. Fixed When converting an auto-assertion to an ExUnit assertion, select the identical pattern when the :default_pattern is :infer (set by default).","ref":"changelog.html#v0-1-3-2023-02-22"},{"type":"extras","title":"Changelog - v0.1.2 (2023-02-21)","doc":"Added Add a :restart option to Mneme.start/1 to restart Mneme if called multiple times.","ref":"changelog.html#v0-1-2-2023-02-21"},{"type":"extras","title":"Changelog - v0.1.1 (2023-02-20)","doc":"Changed Dramatically reduce the performance gap between auto_assert and ExUnit's assert .","ref":"changelog.html#v0-1-1-2023-02-20"},{"type":"extras","title":"Changelog - v0.1.0 (2023-02-19)","doc":"First release.","ref":"changelog.html#v0-1-0-2023-02-19"},{"type":"extras","title":"Under the hood","doc":"To understand how Mneme works, we'll follow the order of execution when running mix test . Afterwards, we'll dive into some specific areas of interest.","ref":"architecture.html"},{"type":"extras","title":"Under the hood - Overview","doc":"Mneme.start/1 is called first. This modifies the ExUnit configuration and starts the Mneme server, a GenServer that coordinates user interaction, manages IO, and stores assertion state. Test modules are loaded and Mneme's macros are expanded: use Mneme registers module attributes used for configuration, like @mneme . auto_assert* macros capture their arguments and environment in %Mneme.Assertion{} structs and output function calls that will run with the test and do the real heavy lifting. Tests begin to run. The Mneme server begins capturing all output from ExUnit, only flushing it to the console when the user is not being prompted. Auto-assertions run as tests are executed. Exactly what this means depends on the type of assertion ( auto_assert , auto_assert_raise , etc.) and whether it is a new assertion with no reference value or not. The basic order of operations is: Capture the value being asserted and compare it against the reference value, if present. If it succeeds, carry on. If the assertion fails or there wasn't a reference value to begin with, a call is made to the Mneme server to update it. A set of patterns are constructed and presented to the user in diffs. The user either selects a pattern or rejects them. If selected, the update is stored for later and the test continues. If rejected, the test fails. Once all tests complete, the Mneme server updates the test files for any patched assertions and prints a summary.","ref":"architecture.html#overview"},{"type":"extras","title":"Under the hood - Generating assertions","doc":"TODO","ref":"architecture.html#generating-assertions"},{"type":"extras","title":"Under the hood - Server responsibilities","doc":"TODO","ref":"architecture.html#server-responsibilities"},{"type":"extras","title":"Under the hood - Terminal UI","doc":"TODO","ref":"architecture.html#terminal-ui"},{"type":"extras","title":"Under the hood - Diffing","doc":"TODO","ref":"architecture.html#diffing"},{"type":"extras","title":"VS Code","doc":"If you are using Visual Studio Code and the ElixirLS extension, there are a couple of things that you can do to smooth out the workflow of running and updating your tests. The ElixirLS extension comes with some built-in tasks to run tests: mix: Run tests mix: Run tests in current file mix: Run test at cursor These are accessible by running the command Run task and then typing typing one of the tasks. By default, these will open a terminal panel, but will not focus it, which isn't optimal as Mneme requires input when something changes. We can bind these tasks to a keyboard shortcut globally and then update the focus defaults on a per-project basis.","ref":"vscode_setup.html"},{"type":"extras","title":"VS Code - Keyboard shortcuts","doc":"Run the command Preferences: Open Keyboard Shortcuts (JSON) , then copy the following keyboard shortcuts to the bottom of the list: { &quot;key&quot; : &quot;ctrl+space a&quot; , &quot;command&quot; : &quot;workbench.action.tasks.runTask&quot; , &quot;args&quot; : &quot;mix: Run tests&quot; } , { &quot;key&quot; : &quot;ctrl+space f&quot; , &quot;command&quot; : &quot;workbench.action.tasks.runTask&quot; , &quot;args&quot; : &quot;mix: Run tests in current file&quot; } , { &quot;key&quot; : &quot;ctrl+space c&quot; , &quot;command&quot; : &quot;workbench.action.tasks.runTask&quot; , &quot;args&quot; : &quot;mix: Run test at cursor&quot; } , You can replace &quot;key&quot; with whatever you'd prefer.","ref":"vscode_setup.html#keyboard-shortcuts"},{"type":"extras","title":"VS Code - Per-project task defaults","doc":"Tasks provided by extensions can be modified by adding a .vscode/tasks.json to your project's root. The following will modify the tasks listed above to give input focus to the panel they're run in. { &quot;version&quot; : &quot;2.0.0&quot; , &quot;tasks&quot; : [ { &quot;group&quot; : &quot;test&quot; , &quot;type&quot; : &quot;mix&quot; , &quot;task&quot; : &quot;Run tests&quot; , &quot;presentation&quot; : { &quot;focus&quot; : true , } , } , { &quot;group&quot; : &quot;test&quot; , &quot;type&quot; : &quot;mix&quot; , &quot;task&quot; : &quot;Run tests in current file&quot; , &quot;presentation&quot; : { &quot;focus&quot; : true , } , } , { &quot;group&quot; : &quot;test&quot; , &quot;type&quot; : &quot;mix&quot; , &quot;task&quot; : &quot;Run test at cursor&quot; , &quot;presentation&quot; : { &quot;focus&quot; : true , } , } , ] , } The Mneme repository has this set up here , for example.","ref":"vscode_setup.html#per-project-task-defaults"}]